/*
 * mcp23017.input.c:
 * This script provide 16 channel input with the MCP23017 with Orange-PI with the interupt port, so only when a button is pressed, the PI will check status all ports 
 * This take much less cpu time then when waiting for the interrupt on all 16 ports.
 *
 * 1. Connect the mcp23017 as usual with i2c, in this example on address 0x20 (3x GND)
 * 2. Connect MCP23017 port 20 (INTA) to port (Physical) 37 on the PI (Number off the Physical pin 37 - Broadcom pin 20 - GPIO.25 - wPi 25)
 * 3. Connect push buttons with GND on MCP23017 port 1-16
 *
 * This example uses the first 8 ports (GPA0-GPA7) as low inputs, so the chip is set to pull up internal, 
 * the last 8 ports (GPB0-GPB7) are for the PIR in my case, they send HIGH when active, 
 * so I have 10K resistors between GND and each GPB0-GPB7 (chip has no internal pull down), 
 * some PIR don't need them, and don't react if placed, some PIR need them because the generate false positive (or being in flooting state). 
 * You can change easy mcp23017.input.c and mcp23017.setup.sh to you're own config.
 *
 * When pushed or detection, it will start a script /root/scripts/mcp23017/mcp23017.GPA0, mcp23017.GPA1, mcp23017.GPB0 etc
 *
 *
 * Created by ILoveIOT 2019 
 ***********************************************************************************
 * For compiling use  : cd /root/scripts/mcp23017 ; gcc -Wall -o mcp23017.input mcp23017.input.c -lwiringPi
 * For console use    : /root/scripts/mcp23017/mcp23017.input
 * For background use : /root/scripts/mcp23017/mcp23017.input &
 ***********************************************************************************
 */

#include <stdio.h>    // Used for printf() statements
#include <wiringPi.h> // Include WiringPi library!
#include <stdlib.h>   // Standard Library
#include <mcp23017.h> // I2C mcp23017 communication
#define Q2W_BASE 100

// We're using the Physical pin number, for the INTA/INTB, Orange-PI GPIO.25 to MCP23017 INTA
const int butPin = 37; // Number off the Physical pin 37 - Broadcom pin 20 - GPIO.25 - wPi 25

int main(void)
{
    //printf("MCP23017 input script is running! Press CTRL+C to quit.\n");
	
	// Add in the mcp23017 on the q2w board
	mcp23017Setup (Q2W_BASE, 0x20) ;

	// Enable the on-goard GPIO
    wiringPiSetup () ;
	
    // Setup stuff:
	wiringPiSetupPhys(); // Initialize wiringPi -- Physical pin numbers
    pinMode(butPin, INPUT); // Set button as INPUT on the PI
    pullUpDnControl(butPin, PUD_UP); // Enable pull-up resistor on button

	// Setup the MCP23017
	system("/bin/bash -c /root/scripts/mcp23017/mcp23017.setup.sh");	
	// Reset the INTA and INTB
	system("/bin/bash -c /usr/sbin/i2cget -y 0 0x20 0x12 > /dev/null 2>&1");
	system("/bin/bash -c /usr/sbin/i2cget -y 0 0x20 0x13 > /dev/null 2>&1");
	
	delay(250);
	
    // Loop (while(1)):
    while(1)
    {
        if (digitalRead(butPin)) // Button is released if this returns 1
        {
			delay(50); // Wait 50ms again
        }
        else // If digitalRead returns 0, button is pressed
        {		
					
		if (digitalRead (Q2W_BASE + 15) == HIGH)  // While Pushed
	    {
			//printf ("GPB7 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPB7.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 14) == HIGH)  // While Pushed
	    {
			//printf ("GPB6 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPB6.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 13) == HIGH)  // While Pushed
	    {
			//printf ("GPB5 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPB5.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 12) == HIGH)  // While Pushed
	    {
			//printf ("GPB4 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPB4.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 11) == HIGH)  // While Pushed
	    {
			//printf ("GPB3 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPB3.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 10) == HIGH)  // While Pushed
	    {
			//printf ("GPB2 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPB2.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 9) == HIGH)  // While Pushed
	    {
			//printf ("GPB1 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPB1.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 8) == HIGH)  // While Pushed
	    {
			//printf ("GPB0 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPB0.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 7) == LOW)  // While Pushed
	    {
			//printf ("GPA7 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPA7.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 6) == LOW)  // While Pushed
	    {
			//printf ("GPA6 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPA6.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 5) == LOW)  // While Pushed
	    {
			//printf ("GPA5 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPA5.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 4) == LOW)  // While Pushed
	    {
			//printf ("GPA4 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPA4.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 3) == LOW)  // While Pushed
	    {
			//printf ("GPA3 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPA3.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 2) == LOW)  // While Pushed
	    {
			//printf ("GPA2 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPA2.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 1) == LOW)  // While Pushed
	    {
			//printf ("GPA1 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPA1.sh &");
        }
		
		if (digitalRead (Q2W_BASE + 0) == LOW)  // While Pushed
	    {
			//printf ("GPA0 is ON\n") ;
			system("/bin/bash -c /root/scripts/mcp23017/mcp23017.GPA0.sh &");
        }
		
		// Reset the INTA and INTB
		system("/bin/bash -c /usr/sbin/i2cget -y 0 0x20 0x12 > /dev/null 2>&1");	
		system("/bin/bash -c /usr/sbin/i2cget -y 0 0x20 0x13 > /dev/null 2>&1");
		delay(50); // Wait 150ms again
        }
    }
    return 0;
}